@@ -3,18 +3,20 @@
 #include <string.h>
 #include <math.h>
 
+#define FUNC_isqrt  1
+#define FUNC_icbrt  1
+#define FUNC_gcd_ui 1
+#define FUNC_is_perfect_square 1
+#define FUNC_clz 1
 #include "ptypes.h"
 #include "factor.h"
 #include "sieve.h"
+#include "util.h"
 #include "mulmod.h"
 #include "cache.h"
 #include "primality.h"
 #include "montmath.h"
-#define FUNC_isqrt  1
-#define FUNC_gcd_ui 1
-#define FUNC_is_perfect_square 1
-#define FUNC_clz 1
-#include "util.h"
+static int holf32(uint32_t n, UV *factors, uint32_t rounds);
 
 /*
  * You need to remember to use UV for unsigned and IV for signed types that
@@ -49,13 +51,10 @@ static const unsigned short primes_small[] =
    1949,1951,1973,1979,1987,1993,1997,1999,2003,2011};
 #define NPRIMES_SMALL (sizeof(primes_small)/sizeof(primes_small[0]))
 
-
-/* The main factoring loop */
-/* Puts factors in factors[] and returns the number found. */
-int factor(UV n, UV *factors)
+static int _small_trial_factor(UV n, UV *factors, UV *newn, uint32_t *lastf)
 {
-  int nfactors = 0;           /* Number of factored in factors result */
-  unsigned int f = 7;
+  int nfactors = 0;
+  uint32_t f = 7;
 
   if (n > 1) {
     while ( (n & 1) == 0 ) { factors[nfactors++] = 2; n /= 2; }
@@ -64,10 +63,11 @@ int factor(UV n, UV *factors)
   }
 
   if (f*f <= n) {
-    UV sp = 4, lastsp = 83;
+    uint32_t const lastsp = 83;
+    uint32_t sp = 4;
     /* Trial division from 7 to 421.  Use 32-bit if possible. */
     if (n <= 4294967295U) {
-      unsigned int un = n;
+      uint32_t un = n;
       while (sp < lastsp) {
         while ( (un%f) == 0 ) {
           factors[nfactors++] = f;
@@ -89,7 +89,7 @@ int factor(UV n, UV *factors)
     }
     /* If n is small and still composite, finish it here */
     if (n < 2011*2011 && f*f <= n) {  /* Trial division from 431 to 2003 */
-      unsigned int un = n;
+      uint32_t un = n;
       while (sp < NPRIMES_SMALL) {
         while ( (un%f) == 0 ) {
           factors[nfactors++] = f;
@@ -101,107 +101,154 @@ int factor(UV n, UV *factors)
       n = un;
     }
   }
-  if (f*f > n) {
-    if (n != 1) factors[nfactors++] = n;
-    return nfactors;
+  if (f*f > n && n != 1) {
+    factors[nfactors++] = n;
+    n = 1;
   }
-#if BITS_PER_WORD == 64   /* this is slower in 32-bit */
+  if (newn) *newn = n;
+  if (lastf) *lastf = f;
+  return nfactors;
+}
+
+static int _power_factor(UV n, UV *factors)
+{
+  int nfactors, i, j, k = powerof(n);
+  if (k > 1) {
+    UV p = rootof(n, k);
+    nfactors = factor(p, factors);
+    for (i = nfactors; i >= 0; i--)
+      for (j = 0; j < k; j++)
+        factors[k*i+j] = factors[i];
+    return k*nfactors;
+  }
+  factors[0] = n;
+  return 1;
+}
+
+/* Find one factor of an input n. */
+int factor_one(UV n, UV *factors, int primality, int trial)
+{
+  int nfactors;
+  if (n < 4) {
+    factors[0] = n;
+    return (n == 1) ? 0 : 1;
+  }
+  /* TODO: deal with small n */
+  if (trial) {
+    uint32_t sp, f;
+    if (!(n&1)) { factors[0] = 2; factors[1] = n >> 1; return 2; }
+    if (!(n%3)) { factors[0] = 3; factors[1] = n / 3; return 2; }
+    if (!(n%5)) { factors[0] = 5; factors[1] = n / 5; return 2; }
+    for (sp = 4; (f = primes_small[sp]) < 2011; sp++) {
+      if ( (n % f) == 0 )
+        { factors[0] = f; factors[1] = n/f; return 2; }
+    }
+    if (n < f*f) { factors[0] = n; return 1; }
+  }
+  if (primality && is_prime(n)) {
+    factors[0] = n;
+    return 1;
+  }
+#if 0  /* Simple solution, just fine on x86_64 */
+  nfactors = (n < 1073741824UL) ? holf32(n, factors, 1000000)
+                                : pbrent_factor(n, factors, 500000, 1);
+  if (nfactors < 2) croak("factor_one failed on %lu\n", n);
+#endif
+  {
+    /* Adjust the number of rounds based on the number size and speed */
+    UV const nbits = BITS_PER_WORD - clz(n);
+#if USE_MONTMATH
+    UV const br_rounds = 8000 + (9000 * ((nbits <= 45) ? 0 : (nbits-45)));
+    UV const sq_rounds = 200000;
+#elif MULMODS_ARE_FAST
+    UV const br_rounds =  500 + ( 200 * ((nbits <= 45) ? 0 : (nbits-45)));
+    UV const sq_rounds = 100000;
+#else
+    UV const br_rounds = (nbits >= 63) ? 120000 : (nbits >= 58) ? 500 : 0;
+    UV const sq_rounds = 200000;
+#endif
+
+#if BITS_PER_WORD == 64
+    /* For small semiprimes the fastest solution is HOLF under 32, then
+     * Lehman (no trial) under 38.  However on random inputs, HOLF is
+     * best only under 28-30 bits, and adding Lehman is always slower. */
+    if (nbits <= 30) { /* This should always succeed */
+      nfactors = holf32(n, factors, 1000000);
+      if (nfactors > 1) return nfactors;
+    }
+#endif
+    /* Almost all inputs are factored here */
+    if (br_rounds > 0) {
+      nfactors = pbrent_factor(n, factors, br_rounds, 1);
+      if (nfactors > 1) return nfactors;
+    }
+#if USE_MONTMATH
+    nfactors = pbrent_factor(n, factors, 2*br_rounds, 3);
+    if (nfactors > 1) return nfactors;
+#endif
+    /* Random 64-bit inputs at this point:
+     *   About 3.1% are small enough that we did with HOLF.
+     *   montmath:  96.89% pbrent,  0.01% pbrent2
+     *   fast:      73.43% pbrent, 21.97% squfof, 1.09% p-1, 0.49% prho, long
+     *   slow:      75.34% squfof, 19.47% pbrent, 0.20% p-1, 0.06% prho
+     */
+    /* SQUFOF with these parameters gets 99.9% of everything left */
+    if (nbits <= 62) {
+      nfactors = squfof_factor(n, factors, sq_rounds);
+      if (nfactors > 1) return nfactors;
+    }
+    /* At this point we should only have 16+ digit semiprimes. */
+    nfactors = pminus1_factor(n, factors, 8000, 120000);
+    if (nfactors > 1) return nfactors;
+    /* Get the stragglers */
+    nfactors = prho_factor(n, factors, 120000);
+    if (nfactors > 1) return nfactors;
+    nfactors = pbrent_factor(n, factors, 500000, 5);
+    if (nfactors > 1) return nfactors;
+    nfactors = prho_factor(n, factors, 120000);
+    if (nfactors > 1) return nfactors;
+    croak("factor_one failed on %lu\n", n);
+  }
+  return nfactors;
+}
+
+/******************************************************************************/
+/*                              Main factor loop                              */
+/*                                                                            */
+/* Puts factors in factors[] and returns the number found.                    */
+/******************************************************************************/
+int factor(UV n, UV *factors)
+{
+  UV tofac_stack[MPU_MAX_FACTORS+1];
+  int nsmallfactors, npowerfactors, nfactors, i, j, ntofac = 0;
+  uint32_t f;
+
+  nfactors = _small_trial_factor(n, factors, &n, &f);
+  if (n == 1) return nfactors;
+
+#if BITS_PER_WORD == 64
   /* For small values less than f^3, use simple factor to split semiprime */
-  if (n < 200000000 && n < f*f*f) {
-    if (is_prob_prime(n)) factors[nfactors++] = n;
-    else                  nfactors += holf_factor(n, factors+nfactors, 1000000);
+  if (n < 100000000 && n < f*f*f) {
+    if (MR32(n)) factors[nfactors++] = n;
+    else         nfactors += holf32(n, factors+nfactors, 10000);
     return nfactors;
   }
 #endif
 
-  /* Perfect powers.  Factor root only once. */
-  {
-    int i, j, k = powerof(n);
-    if (k > 1) {
-      UV p = rootof(n, k);
-      int nsmallfactors = nfactors;
-      nfactors = factor(p, factors+nsmallfactors);
-      for (i = nfactors; i >= 0; i--)
-        for (j = 0; j < k; j++)
-          factors[nsmallfactors+k*i+j] = factors[nsmallfactors+i];
-      return nsmallfactors + k*nfactors;
-    }
-  }
+  nsmallfactors = nfactors;
 
-  {
-  UV tofac_stack[MPU_MAX_FACTORS+1];
-  int i, j, ntofac = 0;
-  int nsmallfactors = nfactors;
-  int const verbose = _XS_get_verbose();
+  /* Perfect powers.  Factor root only once. */
+  npowerfactors = _power_factor(n, factors+nsmallfactors);
+  if (npowerfactors > 1) return nsmallfactors + npowerfactors;
 
   /* loop over each remaining factor, until ntofac == 0 */
   do {
-    while ( (n >= f*f) && (!is_prob_prime(n)) ) {
-      int split_success = 0;
-      /* Adjust the number of rounds based on the number size and speed */
-#if USE_MONTMATH
-      UV const br_rounds = ((n>>29)<100000) ?  8000 : 80000;
-#elif MULMODS_ARE_FAST
-      UV const br_rounds = ((n>>29)<100000) ?   500 :  2000;
-#else
-      UV const br_rounds = (n < (UV_MAX>>2))?     0 : 10000;
-#endif
-      UV const sq_rounds = 200000; /* 20k 91%, 40k 98%, 80k 99.9%, 120k 99.99%*/
-
-      /* 99.7% of 32-bit, 94% of 64-bit random inputs factored here */
-      if (!split_success && br_rounds > 0) {
-        split_success = pbrent_factor(n, tofac_stack+ntofac, br_rounds, 3)-1;
-        if (verbose) { if (split_success) printf("pbrent 1:  %"UVuf" %"UVuf"\n", tofac_stack[ntofac], tofac_stack[ntofac+1]); else printf("pbrent 0\n"); }
-      }
-      /* Give larger inputs a run with p-1 before SQUFOF */
-      if (!split_success && n > (UV_MAX >> 15) && MULMODS_ARE_FAST) {
-        split_success = pminus1_factor(n, tofac_stack+ntofac, 1000, 15000)-1;
-        if (verbose) printf("small p-1 %d\n", split_success);
-      }
-      /* SQUFOF with these parameters gets 99.9% of everything left */
-      if (!split_success && n < (UV_MAX>>2)) {
-        split_success = squfof_factor(n,tofac_stack+ntofac, sq_rounds)-1;
-        if (verbose) printf("squfof %d\n", split_success);
-      }
-      /* At this point we should only have 16+ digit semiprimes. */
-      if (!split_success) {
-        split_success = pminus1_factor(n, tofac_stack+ntofac, 8000, 120000)-1;
-        if (verbose) printf("pminus1 %d\n", split_success);
-        /* Get the stragglers */
-        if (!split_success) {
-          split_success = prho_factor(n, tofac_stack+ntofac, 120000)-1;
-          if (verbose) printf("long prho %d\n", split_success);
-          if (!split_success) {
-            split_success = pbrent_factor(n, tofac_stack+ntofac, 500000, 7)-1;
-            if (verbose) printf("long pbrent %d\n", split_success);
-          }
-        }
-      }
-
-      if (split_success) {
-        MPUassert( split_success == 1, "split factor returned more than 2 factors");
-        ntofac++; /* Leave one on the to-be-factored stack */
-        if ((tofac_stack[ntofac] == n) || (tofac_stack[ntofac] == 1))
-          croak("bad factor\n");
-        n = tofac_stack[ntofac];  /* Set n to the other one */
-      } else {
-        /* Factor via trial division.  Nothing should ever get here. */
-        UV m = f % 30;
-        UV limit = isqrt(n);
-        if (verbose) printf("doing trial on %"UVuf"\n", n);
-        while (f <= limit) {
-          if ( (n%f) == 0 ) {
-            do {
-              n /= f;
-              factors[nfactors++] = f;
-            } while ( (n%f) == 0 );
-            limit = isqrt(n);
-          }
-          f += wheeladvance30[m];
-          m =  nextwheel30[m];
-        }
-        break;  /* We just factored n via trial division.  Exit loop. */
-      }
+    while ( (n >= f*f) && (!is_def_prime(n)) ) {
+      int split_success = factor_one(n, tofac_stack+ntofac, 0, 0) - 1;
+      if (split_success != 1 || tofac_stack[ntofac] == 1 || tofac_stack[ntofac] == n)
+        croak("internal: factor_one failed to factor %"UVuf"\n", n);
+      ntofac++; /* Leave one on the to-be-factored stack */
+      n = tofac_stack[ntofac];  /* Set n to the other one */
     }
     /* n is now prime (or 1), so add to already-factored stack */
     if (n != 1)  factors[nfactors++] = n;
@@ -216,7 +263,6 @@ int factor(UV n, UV *factors)
       factors[j] = factors[j-1];
     factors[j] = fi;
   }
-  }
   return nfactors;
 }
 
@@ -246,50 +292,49 @@ int factor_exp(UV n, UV *factors, UV* exponents)
   return j;
 }
 
-
-int trial_factor(UV n, UV *factors, UV maxtrial)
+int trial_factor(UV n, UV *factors, UV f, UV last)
 {
-  int nfactors = 0;
+  int sp, nfactors = 0;
 
-  if (maxtrial == 0)  maxtrial = UV_MAX;
+  if (f < 2) f = 2;
+  if (last == 0 || last*last > n) last = UV_MAX;
 
-  /* Cover the cases 0/1/2/3 now */
-  if (n < 4 || maxtrial < 2) {
+  if (n < 4 || last < f) {
     factors[0] = n;
     return (n == 1) ? 0 : 1;
   }
-  /* Trial division for 2, 3, 5 immediately */
-  while ( (n & 1) == 0 ) { factors[nfactors++] = 2; n /= 2; }
-  if (3<=maxtrial) while ( (n % 3) == 0 ) { factors[nfactors++] = 3; n /= 3; }
-  if (5<=maxtrial) while ( (n % 5) == 0 ) { factors[nfactors++] = 5; n /= 5; }
 
-  if (7*7 <= n) {
-    UV f, sp = 3;
-    while (++sp < NPRIMES_SMALL) {
+  /* possibly do uint32_t specific code here */
+
+  if (f < primes_small[NPRIMES_SMALL-1]) {
+    while ( (n & 1) == 0 ) { factors[nfactors++] = 2; n >>= 1; }
+    if (3<=last) while ( (n % 3) == 0 ) { factors[nfactors++] = 3; n /= 3; }
+    if (5<=last) while ( (n % 5) == 0 ) { factors[nfactors++] = 5; n /= 5; }
+    for (sp = 4; sp < (int)NPRIMES_SMALL; sp++) {
       f = primes_small[sp];
-      if (f*f > n || f > maxtrial) break;
+      if (f*f > n || f > last) break;
       while ( (n%f) == 0 ) {
         factors[nfactors++] = f;
         n /= f;
       }
     }
-    /* Trial division using a mod-30 wheel for larger values */
-    if (f*f <= n && f <= maxtrial) {
-      UV m, newlimit, limit = isqrt(n);
-      if (limit > maxtrial) limit = maxtrial;
-      m = f % 30;
-      while (f <= limit) {
-        if ( (n%f) == 0 ) {
-          do {
-            factors[nfactors++] = f;
-            n /= f;
-          } while ( (n%f) == 0 );
-          newlimit = isqrt(n);
-          if (newlimit < limit)  limit = newlimit;
-        }
-        f += wheeladvance30[m];
-        m = nextwheel30[m];
+  }
+  /* Trial division using a mod-30 wheel for larger values */
+  if (f*f <= n && f <= last) {
+    UV m, newlimit, limit = isqrt(n);
+    if (limit > last) limit = last;
+    m = f % 30;
+    while (f <= limit) {
+      if ( (n%f) == 0 ) {
+        do {
+          factors[nfactors++] = f;
+          n /= f;
+        } while ( (n%f) == 0 );
+        newlimit = isqrt(n);
+        if (newlimit < limit)  limit = newlimit;
       }
+      f += wheeladvance30[m];
+      m = nextwheel30[m];
     }
   }
   /* All done! */
@@ -313,9 +358,6 @@ static void _divisors_from_factors(UV nfactors, UV* fp, UV* fe, UV* res) {
   }
 }
 
-static int numcmp(const void *a, const void *b)
-  { const UV *x = a, *y = b; return (*x > *y) ? 1 : (*x < *y) ? -1 : 0; }
-
 UV* _divisor_list(UV n, UV *num_divisors)
 {
   UV factors[MPU_MAX_FACTORS+1];
@@ -338,7 +380,7 @@ UV* _divisor_list(UV n, UV *num_divisors)
   New(0, divs, ndivisors, UV);
   _divisors_from_factors(nfactors, factors, exponents, divs);
   /* Sort divisors (numeric ascending) */
-  qsort(divs, ndivisors, sizeof(UV), numcmp);
+  qsort(divs, ndivisors, sizeof(UV), _numcmp);
   /* Return number of divisors and list */
   *num_divisors = ndivisors;
   return divs;
@@ -421,8 +463,7 @@ static int found_factor(UV n, UV f, UV* factors)
 }
 
 /* Knuth volume 2, algorithm C.
- * Very fast for small numbers, grows rapidly.
- * SQUFOF is better for numbers nearing the 64-bit limit.
+ * Can't compete with HOLF, SQUFOF, pbrent, etc.
  */
 int fermat_factor(UV n, UV *factors, UV rounds)
 {
@@ -453,12 +494,18 @@ int holf_factor(UV n, UV *factors, UV rounds)
 
   MPUassert( (n >= 3) && ((n%2) != 0) , "bad n in holf_factor");
 
+  /* We skip the perfect-square test for s in the loop, so we
+   * will never succeed if n is a perfect square.  Test that now. */
+  if (is_perfect_square(n))
+    return found_factor(n, isqrt(n), factors);
+
   if (n <= (UV_MAX >> 6)) {    /* Try with premultiplier first */
     UV npre = n * ( (n <= (UV_MAX >> 13)) ? 720 :
                     (n <= (UV_MAX >> 11)) ? 480 :
                     (n <= (UV_MAX >> 10)) ? 360 :
                     (n <= (UV_MAX >>  8)) ?  60 : 30 );
     UV ni = npre;
+#if 0                              /* Straightforward */
     while (rounds--) {
       s = isqrt(ni) + 1;
       m = (s*s) - ni;
@@ -470,6 +517,23 @@ int holf_factor(UV n, UV *factors, UV rounds)
       if (ni >= (ni+npre)) break;
       ni += npre;
     }
+#else                              /* More optimized */
+    while (rounds--) {
+      s = 1 + (UV)sqrt((double)ni);
+      m = (s*s) - ni;
+      f = m & 127;
+      if (!((f*0x8bc40d7d) & (f*0xa1e2f5d1) & 0x14020a)) {
+        f = (UV)sqrt((double)m);
+        if (m == f*f) {
+          f = gcd_ui(n, s - f);
+          if (f > 1 && f < n)
+            return found_factor(n, f, factors);
+        }
+      }
+      if (ni >= (ni+npre)) break;
+      ni += npre;
+    }
+#endif
   }
 
   for (i = 1; i <= rounds; i++) {
@@ -488,37 +552,84 @@ int holf_factor(UV n, UV *factors, UV rounds)
   factors[0] = n;
   return 1;
 }
+static int holf32(uint32_t n, UV *factors, uint32_t rounds) {
+  UV npre, ni;    /* These should be 64-bit */
+  uint32_t s, m, f;
+
+  if (n < 3) { factors[0] = n; return 1; }
+  if (!(n&1)) { factors[0] = 2; factors[1] = n/2; return 2; }
+  if (is_perfect_square(n)) { factors[0] = factors[1] = isqrt(n); return 2; }
+
+  ni = npre = (UV) n * ((BITS_PER_WORD == 64) ? 5040 : 1);
+  while (rounds--) {
+    s = 1 + (uint32_t)sqrt((double)ni);
+    m = ((UV)s*(UV)s) - ni;
+    f = m & 127;
+    if (!((f*0x8bc40d7d) & (f*0xa1e2f5d1) & 0x14020a)) {
+      f = (uint32_t)sqrt((double)m);
+      if (m == f*f) {
+        f = gcd_ui(n, s - f);
+        if (f > 1 && f < n)
+          return found_factor(n, f, factors);
+      }
+    }
+    if (ni >= (ni+npre)) break; /* We've overflowed */
+    ni += npre;
+  }
+  factors[0] = n;
+  return 1;
+}
 
 
+#define ABSDIFF(x,y)  (x>y) ? x-y : y-x
 #if USE_MONTMATH
-/* Pollard / Brent.  Brent's modifications to Pollard's Rho.  Maybe faster. */
+/* Pollard Rho with Brent's updates, using Montgomery reduction. */
 int pbrent_factor(UV n, UV *factors, UV rounds, UV a)
 {
-  UV f, m, r, Xi, Xm;
-  const UV inner = (n <= 4000000000UL) ? 32 : 160;
-  int fails = 6;
+  UV const nbits = BITS_PER_WORD - clz(n);
+  const UV inner = (nbits <= 31) ? 32 : (nbits <= 35) ? 64 : (nbits <= 40) ? 160 : (nbits <= 52) ? 256 : 320;
+  UV f, m, r, rleft, Xi, Xm, Xs;
+  int irounds, fails = 6;
   const uint64_t npi = mont_inverse(n),  mont1 = mont_get1(n);
-  Xi = Xm = mont_get2(n);
-  a = mont_geta(a,n);
-  MPUassert( (n >= 3) && ((n%2) != 0) , "bad n in pbrent_factor");
 
-  r = 1;
-  f = 1;
+  MPUassert( (n >= 3) && ((n%2) != 0) , "bad n in pbrent_factor");
+  r = f = 1;
+  Xi = Xm = Xs = mont1;
+  a = mont_geta(a,n);
+
   while (rounds > 0) {
-    UV rleft = (r > rounds) ? rounds : r;
-    UV saveXi = Xi;
+    rleft = (r > rounds) ? rounds : r;
+    Xm = Xi;
     /* Do rleft rounds, inner at a time */
     while (rleft > 0) {
-      UV dorounds = (rleft > inner) ? inner : rleft;
-      saveXi = Xi;
-      rleft -= dorounds;
-      rounds -= dorounds;
-      Xi = mont_sqrmod(Xi,n);  Xi = addmod(Xi,a,n);
-      m = (Xi>Xm) ? Xi-Xm : Xm-Xi;
-      while (--dorounds > 0) {         /* Now do inner-1=63 more iterations */
-        Xi = mont_sqrmod(Xi,n);  Xi = addmod(Xi,a,n);
-        f = (Xi>Xm) ? Xi-Xm : Xm-Xi;
-        m = mont_mulmod(m, f, n);
+      irounds = (rleft > (UV)inner) ? inner : rleft;
+      rleft -= irounds;
+      rounds -= irounds;
+      Xs = Xi;
+      if (n < (1ULL << 63)) {
+        Xi = mont_mulmod63(Xi,Xi+a,n);
+        m = ABSDIFF(Xi,Xm);
+        while (--irounds > 0) {
+          Xi = mont_mulmod63(Xi,Xi+a,n);
+          f = ABSDIFF(Xi,Xm);
+          m = mont_mulmod63(m, f, n);
+        }
+      } else if (a == mont1) {
+        Xi = mont_mulmod64(Xi,Xi+a,n);
+        m = ABSDIFF(Xi,Xm);
+        while (--irounds > 0) {
+          Xi = mont_mulmod64(Xi,Xi+a,n);
+          f = ABSDIFF(Xi,Xm);
+          m = mont_mulmod64(m, f, n);
+        }
+      } else {
+        Xi = addmod(mont_mulmod64(Xi,Xi,n), a, n);
+        m = ABSDIFF(Xi,Xm);
+        while (--irounds > 0) {
+          Xi = addmod(mont_mulmod64(Xi,Xi,n), a, n);
+          f = ABSDIFF(Xi,Xm);
+          m = mont_mulmod64(m, f, n);
+        }
       }
       f = gcd_ui(m, n);
       if (f != 1)
@@ -527,21 +638,23 @@ int pbrent_factor(UV n, UV *factors, UV rounds, UV a)
     /* If f == 1, then we didn't find a factor.  Move on. */
     if (f == 1) {
       r *= 2;
-      Xm = Xi;
       continue;
     }
     if (f == n) {  /* back up, with safety */
-      Xi = saveXi;
+      Xi = Xs;
       do {
-        Xi = mont_sqrmod(Xi,n);  Xi = addmod(Xi,a,n);
-        f = gcd_ui( (Xi>Xm) ? Xi-Xm : Xm-Xi, n);
+        if (n < (1ULL << 63) || a == mont1)
+          Xi = mont_mulmod(Xi,Xi+a,n);
+        else
+          Xi = addmod(mont_mulmod(Xi,Xi,n),a,n);
+        m = ABSDIFF(Xi,Xm);
+        f = gcd_ui(m, n);
       } while (f == 1 && r-- != 0);
     }
     if (f == 0 || f == n) {
       if (fails-- <= 0) break;
-      Xm = addmod(Xm, 2, n);
-      Xi = Xm;
-      a++;
+      Xi = Xm = mont1;
+      a = addmod(a, mont_geta(11,n), n);
       continue;
     }
     return found_factor(n, f, factors);
@@ -550,19 +663,16 @@ int pbrent_factor(UV n, UV *factors, UV rounds, UV a)
   return 1;
 }
 #else
-/* Pollard / Brent.  Brent's modifications to Pollard's Rho.  Maybe faster. */
+/* Pollard Rho with Brent's updates. */
 int pbrent_factor(UV n, UV *factors, UV rounds, UV a)
 {
-  UV f, m, r;
-  UV Xi = 2;
-  UV Xm = 2;
+  UV f, m, r, Xi, Xm;
   const UV inner = (n <= 4000000000UL) ? 32 : 160;
   int fails = 6;
 
   MPUassert( (n >= 3) && ((n%2) != 0) , "bad n in pbrent_factor");
 
-  r = 1;
-  f = 1;
+  r = f = Xi = Xm = 1;
   while (rounds > 0) {
     UV rleft = (r > rounds) ? rounds : r;
     UV saveXi = Xi;
@@ -573,10 +683,10 @@ int pbrent_factor(UV n, UV *factors, UV rounds, UV a)
       rleft -= dorounds;
       rounds -= dorounds;
       Xi = sqraddmod(Xi, a, n);        /* First iteration, no mulmod needed */
-      m = (Xi>Xm) ? Xi-Xm : Xm-Xi;
+      m = ABSDIFF(Xi,Xm);
       while (--dorounds > 0) {         /* Now do inner-1=63 more iterations */
         Xi = sqraddmod(Xi, a, n);
-        f = (Xi>Xm) ? Xi-Xm : Xm-Xi;
+        f = ABSDIFF(Xi,Xm);
         m = mulmod(m, f, n);
       }
       f = gcd_ui(m, n);
@@ -593,12 +703,12 @@ int pbrent_factor(UV n, UV *factors, UV rounds, UV a)
       Xi = saveXi;
       do {
         Xi = sqraddmod(Xi, a, n);
-        f = gcd_ui( (Xi>Xm) ? Xi-Xm : Xm-Xi, n);
+        f = gcd_ui( ABSDIFF(Xi,Xm), n);
       } while (f == 1 && r-- != 0);
     }
     if (f == 0 || f == n) {
       if (fails-- <= 0) break;
-      Xm = addmod(Xm, 2, n);
+      Xm = addmod(Xm, 11, n);
       Xi = Xm;
       a++;
       continue;
@@ -675,6 +785,15 @@ int pminus1_factor(UV n, UV *factors, UV B1, UV B2)
   UV savea = 2, saveq = 2;
   UV j = 1;
   UV sqrtB1 = isqrt(B1);
+#if USE_MONTMATH
+  const uint64_t npi = mont_inverse(n),  mont1 = mont_get1(n);
+  UV ma = mont_geta(a,n);
+  #define PMINUS1_APPLY_POWER  ma = mont_powmod(ma, k, n)
+  #define PMINUS1_RECOVER_A    a = mont_recover(ma,n)
+#else
+  #define PMINUS1_APPLY_POWER  a = powmod(a, k, n)
+  #define PMINUS1_RECOVER_A
+#endif
   MPUassert( (n >= 3) && ((n%2) != 0) , "bad n in pminus1_factor");
 
   if (B1 <= primes_small[NPRIMES_SMALL-2]) {
@@ -685,13 +804,15 @@ int pminus1_factor(UV n, UV *factors, UV B1, UV B2)
         k = q*q;  kmin = B1/q;
         while (k <= kmin)  k *= q;
       }
-      a = powmod(a, k, n);
+      PMINUS1_APPLY_POWER;
       if ( (j++ % 32) == 0) {
+        PMINUS1_RECOVER_A;
         if (a == 0 || gcd_ui(a-1, n) != 1)
           break;
         savea = a;  saveq = q;
       }
     }
+    PMINUS1_RECOVER_A;
   } else {
     START_DO_FOR_EACH_PRIME(2, B1) {
       q = k = p;
@@ -699,13 +820,15 @@ int pminus1_factor(UV n, UV *factors, UV B1, UV B2)
         k = q*q;  kmin = B1/q;
         while (k <= kmin)  k *= q;
       }
-      a = powmod(a, k, n);
+      PMINUS1_APPLY_POWER;
       if ( (j++ % 32) == 0) {
+        PMINUS1_RECOVER_A;
         if (a == 0 || gcd_ui(a-1, n) != 1)
           break;
         savea = a;  saveq = q;
       }
     } END_DO_FOR_EACH_PRIME
+    PMINUS1_RECOVER_A;
   }
   if (a == 0) { factors[0] = n; return 1; }
   f = gcd_ui(a-1, n);
@@ -836,23 +959,32 @@ int pplus1_factor(UV n, UV *factors, UV B1)
 
 
 /* SQUFOF, based on Ben Buhrow's racing version. */
-
+#if 1
+  /* limit to 62-bit inputs, use 32-bit types, faster */
+  #define SQUFOF_TYPE uint32_t
+  #define SQUFOF_MAX  (UV_MAX >> 2)
+#else
+  /* All 64-bit inputs possible, though we severely limit multipliers */
+  #define SQUFOF_TYPE UV
+  #define SQUFOF_MAX  UV_MAX
+#endif
 typedef struct
 {
   int valid;
-  UV P;
-  UV bn;
-  UV Qn;
-  UV Q0;
-  UV b0;
-  UV it;
-  UV imax;
+  SQUFOF_TYPE P;
+  SQUFOF_TYPE bn;
+  SQUFOF_TYPE Qn;
+  SQUFOF_TYPE Q0;
+  SQUFOF_TYPE b0;
+  SQUFOF_TYPE it;
+  SQUFOF_TYPE imax;
+  SQUFOF_TYPE mult;
 } mult_t;
 
 /* N < 2^63 (or 2^31).  Returns 0 or a factor */
 static UV squfof_unit(UV n, mult_t* mult_save)
 {
-  UV imax,i,Q0,b0,Qn,bn,P,bbn,Ro,S,So,t1,t2;
+  SQUFOF_TYPE imax,i,Q0,Qn,bn,b0,P,bbn,Ro,S,So,t1,t2;
 
   P = mult_save->P;
   bn = mult_save->bn;
@@ -892,19 +1024,22 @@ static UV squfof_unit(UV n, mult_t* mult_save)
       SQUARE_SEARCH_ITERATION;
 
       /* Even iteration.  Check for square: Qn = S*S */
-      if (is_perfect_square(Qn))
-        break;
+      t2 = Qn & 127;
+      if (!((t2*0x8bc40d7d) & (t2*0xa1e2f5d1) & 0x14020a)) {
+        t1 = (uint32_t) sqrt(Qn);
+        if (Qn == t1*t1)
+          break;
+      }
 
       /* Odd iteration. */
       SQUARE_SEARCH_ITERATION;
     }
-    S = isqrt(Qn);
-    /* printf("found square %"UVuf" after %"UVuf" iterations with mult %d\n", Qn, i, mult_save->mult); */
+    S = t1; /* isqrt(Qn); */
+    mult_save->it = i;
 
     /* Reduce to G0 */
     Ro = P + S*((b0 - P)/S);
-    t1 = Ro;
-    So = (n - t1*t1)/S;
+    So = (n - (UV)Ro*(UV)Ro)/(UV)S;
     bbn = (b0+Ro)/So;
 
     /* Search for symmetry point */
@@ -952,75 +1087,300 @@ static const UV squfof_multipliers[] =
 
 int squfof_factor(UV n, UV *factors, UV rounds)
 {
-  const UV big2 = UV_MAX;
   mult_t mult_save[NSQUFOF_MULT];
-  int still_racing;
-  UV i, nn64, mult, f64;
-  UV rounds_done = 0;
+  UV i, nn64, sqrtnn64, mult, f64,rounds_done = 0;
+  int mults_racing = NSQUFOF_MULT;
 
   /* Caller should have handled these trivial cases */
   MPUassert( (n >= 3) && ((n%2) != 0) , "bad n in squfof_factor");
 
   /* Too big */
-  if (n > big2) {
+  if (n > SQUFOF_MAX) {
     factors[0] = n;  return 1;
   }
 
-  for (i = 0; i < NSQUFOF_MULT; i++)
+  for (i = 0; i < NSQUFOF_MULT; i++) {
     mult_save[i].valid = -1;
+    mult_save[i].it = 0;
+  }
 
-  /* Process the multipliers a little at a time: 0.33*(n*mult)^1/4: 20-20k */
-  do {
-    still_racing = 0;
-    for (i = 0; i < NSQUFOF_MULT; i++) {
+  /* Race each multiplier for a bit (20-20k rounds) */
+  while (mults_racing > 0 && rounds_done < rounds) {
+    for (i = 0; i < NSQUFOF_MULT && rounds_done < rounds; i++) {
       if (mult_save[i].valid == 0)  continue;
       mult = squfof_multipliers[i];
       nn64 = n * mult;
       if (mult_save[i].valid == -1) {
-        if ((big2 / mult) < n) {
+        if ((SQUFOF_MAX / mult) < n) {
           mult_save[i].valid = 0; /* This multiplier would overflow 64-bit */
+          mults_racing--;
           continue;
         }
+        sqrtnn64 = isqrt(nn64);
         mult_save[i].valid = 1;
-        mult_save[i].b0 = isqrt(nn64);
-        mult_save[i].imax = (UV) (sqrt(mult_save[i].b0) / 16);
+        mult_save[i].Q0    = 1;
+        mult_save[i].b0    = sqrtnn64;
+        mult_save[i].P     = sqrtnn64;
+        mult_save[i].Qn    = (SQUFOF_TYPE)(nn64 - sqrtnn64 * sqrtnn64);
+        if (mult_save[i].Qn == 0)
+          return found_factor(n, sqrtnn64, factors);
+        mult_save[i].bn    = (2 * sqrtnn64) / (UV)mult_save[i].Qn;
+        mult_save[i].it    = 0;
+        mult_save[i].mult  = mult;
+        mult_save[i].imax  = (UV) (sqrt(sqrtnn64) / 16);
         if (mult_save[i].imax < 20)     mult_save[i].imax = 20;
         if (mult_save[i].imax > rounds) mult_save[i].imax = rounds;
-        mult_save[i].Q0 = 1;
-        mult_save[i].P  = mult_save[i].b0;
-        mult_save[i].Qn = nn64 - (mult_save[i].b0 * mult_save[i].b0);
-        if (mult_save[i].Qn == 0) {
-          factors[0] = mult_save[i].b0;
-          factors[1] = n / mult_save[i].b0;
-          MPUassert( factors[0] * factors[1] == n , "incorrect factoring");
-          return 2;
-        }
-        mult_save[i].bn = (mult_save[i].b0 + mult_save[i].P) / mult_save[i].Qn;
-        mult_save[i].it = 0;
       }
+      if (mults_racing == 1)  /* Do all rounds if only one multiplier left */
+        mult_save[i].imax = (rounds - rounds_done);
       f64 = squfof_unit(nn64, &mult_save[i]);
       if (f64 > 1) {
-        if (f64 != mult) {
-          f64 /= gcd_ui(f64, mult);
-          if (f64 != 1)
-            return found_factor(n, f64, factors);
+        UV f64red = f64 / gcd_ui(f64, mult);
+        if (f64red > 1) {
+          /* unsigned long totiter = 0;
+             {int K; for (K = 0; K < NSQUFOF_MULT; K++) totiter += mult_save[K].it; }
+             printf("  n %lu mult %lu it %lu (%lu)\n",n,mult,totiter,(UV)mult_save[i].it); */
+          return found_factor(n, f64red, factors);
         }
         /* Found trivial factor.  Quit working with this multiplier. */
         mult_save[i].valid = 0;
       }
-      if (mult_save[i].valid == 1)
-        still_racing = 1;
-      rounds_done += mult_save[i].imax;
-      if (rounds_done >= rounds)
-        break;
+      if (mult_save[i].valid == 0)
+        mults_racing--;
+      rounds_done += mult_save[i].imax;   /* Assume we did all rounds */
     }
-  } while (still_racing && rounds_done < rounds);
+  }
 
   /* No factors found */
   factors[0] = n;
   return 1;
 }
 
+#define SQR_TAB_SIZE 512
+static int sqr_tab_init = 0;
+static double sqr_tab[SQR_TAB_SIZE];
+static void make_sqr_tab(void) {
+  int i;
+  for (i = 0; i < SQR_TAB_SIZE; i++)
+    sqr_tab[i] = sqrt((double)i);
+  sqr_tab_init = 1;
+}
+
+/* Lehman written and tuned by Warren D. Smith.
+ * Revised by Ben Buhrow and Dana Jacobsen. */
+int lehman_factor(UV n, UV *factors, int do_trial) {
+  const double Tune = ((n >> 31) >> 5) ? 3.5 : 5.0;
+  double x, sqrtn;
+  UV a,c,kN,kN4,B2;
+  uint32_t b,p,k,r,B,U,Bred,inc,ip=2;
+
+  if (!(n&1)) return found_factor(n, 2, factors);
+
+  B = Tune * (1+rootof(n,3));
+
+  if (do_trial) {
+    uint32_t FirstCut = 0.1 * B;
+    if (FirstCut < 84) FirstCut = 84;
+    if (FirstCut > 65535) FirstCut = 65535;
+    for (ip = 2;  ip < NPRIMES_SMALL; ip++) {
+      p = primes_small[ip];
+      if (p >= FirstCut)
+        break;
+      if (n % p == 0)
+        return found_factor(n, p, factors);
+    }
+  }
+
+  if (n >= UVCONST(8796393022207)) { factors[0] = n; return 1; }
+  Bred = B / (Tune * Tune * Tune);
+  B2 = B*B;
+  kN = 0;
+
+  if (!sqr_tab_init) make_sqr_tab();
+  sqrtn = sqrt(n);
+
+  for (k = 1; k <= Bred; k++) {
+    if (k&1) { inc = 4; r = (k+n) % 4; }
+    else     { inc = 2; r = 1; }
+    kN += n;
+    if (kN >= UVCONST(1152921504606846976)) { factors[0] = n; return 1; }
+    kN4 = kN*4;
+
+    x = (k < SQR_TAB_SIZE) ? sqrtn * sqr_tab[k] : sqrt((double)kN);
+    a = x;
+    if ((UV)a * (UV)a == kN)
+      return found_factor(n, gcd_ui(a,n), factors);
+    x *= 2;
+    a = x + 0.9999999665;  /* Magic constant */
+    b = a % inc;
+    b = a + (inc+r-b) % inc;
+    c = (UV)b*(UV)b - kN4;
+    U = x + B2/(2*x);
+    for (a = b;  a <= U;  c += inc*(a+a+inc), a += inc) {
+      /* Check for perfect square */
+      b = c & 127;
+      if (!((b*0x8bc40d7d) & (b*0xa1e2f5d1) & 0x14020a)) {
+        b = (uint32_t) sqrt(c);
+        if (c == b*b) {
+          B2 = gcd_ui(a+b, n);
+          return found_factor(n, B2, factors);
+        }
+      }
+    }
+  }
+  if (do_trial) {
+    if (B > 65535) B = 65535;
+    /* trial divide from primes[ip] to B.  We could:
+     *   1) use table of 6542 shorts for the primes.
+     *   2) use a wheel
+     *   3) let trial_factor handle it
+     */
+    if (ip >= NPRIMES_SMALL)  ip = NPRIMES_SMALL-1;
+    return trial_factor(n, factors, primes_small[ip], B);
+  }
+  factors[0] = n;
+  return 1;
+}
+
+static const uint32_t _fr_chunk = 8192;
+static const uint32_t _fr_sieve_crossover = 10000000;  /* About 10^14 */
+
+static void _vec_factor(UV lo, UV hi, UV *nfactors, UV *farray, UV noffset, int square_free)
+{
+  UV *N, j, n, sqrthi, sievelim;
+  sqrthi = isqrt(hi);
+  n = hi-lo+1;
+  New(0, N, hi-lo+1, UV);
+  for (j = 0; j < n; j++) {
+    N[j] = 1;
+    nfactors[j] = 0;
+  }
+  sievelim = (sqrthi < _fr_sieve_crossover) ? sqrthi : icbrt(hi);
+  START_DO_FOR_EACH_PRIME(2, sievelim) {
+    UV q, t, A;
+    if (square_free == 0) {
+      UV kmin = hi / p;
+      for (q = p; q <= kmin; q *= p) {
+        t = lo / q, A = t * q;
+        if (A < lo) A += q;
+        for (j = A-lo; j < n; j += q) {
+          farray[ j*noffset + nfactors[j]++ ] = p;
+          N[j] *= p;
+        }
+      }
+    } else {
+      q = p*p, t = lo / q, A = t * q;
+      if (A < lo) A += q;
+      for (j = A-lo; j < n; j += q) {
+        N[j] = 0;
+        nfactors[j] = 0;
+      }
+      q = p, t = lo / q, A = t * q;
+      if (A < lo) A += q;
+      for (j = A-lo; j < n; j += q) {
+        if (N[j] > 0) {
+          farray[ j*noffset + nfactors[j]++ ] = p;
+          N[j] *= p;
+        }
+      }
+    }
+  } END_DO_FOR_EACH_PRIME
+
+  if (sievelim == sqrthi) {
+    /* Handle the unsieved results, which are prime */
+    for (j = 0; j < n; j++) {
+      if (N[j] == 1)
+        farray[ j*noffset + nfactors[j]++ ] = j+lo;
+      else if (N[j] > 0 && N[j] != j+lo)
+        farray[ j*noffset + nfactors[j]++ ] = (j+lo) / N[j];
+    }
+  } else {
+    /* Handle the unsieved results, which are prime or semi-prime */
+    for (j = 0; j < n; j++) {
+      UV rem = j+lo;
+      if (N[j] > 0 && N[j] != rem) {
+        if (N[j] != 1)
+          rem /= N[j];
+        if (square_free && is_perfect_square(rem)) {
+          nfactors[j] = 0;
+        } else {
+          UV* f = farray + j*noffset + nfactors[j];
+          nfactors[j] += factor_one(rem, f, 1, 0);
+        }
+      }
+    }
+  }
+  Safefree(N);
+}
+
+factor_range_context_t factor_range_init(UV lo, UV hi, int square_free) {
+  factor_range_context_t ctx;
+  ctx.lo = lo;
+  ctx.hi = hi;
+  ctx.n = lo-1;
+  ctx.is_square_free = square_free ? 1 : 0;
+  if (hi-lo+1 > 100) {        /* Sieve in chunks */
+    if (square_free) ctx._noffset = (hi <= 42949672965UL) ? 10 : 15;
+    else             ctx._noffset = BITS_PER_WORD - clz(hi);
+    ctx._coffset = _fr_chunk;
+    New(0, ctx._nfactors, _fr_chunk, UV);
+    New(0, ctx._farray, _fr_chunk * ctx._noffset, UV);
+    { /* Prealloc all the sieving primes now. */
+      UV t = isqrt(hi);
+      if (t >= _fr_sieve_crossover) t = icbrt(hi);
+      get_prime_cache(t, 0);
+    }
+  } else {                    /* factor each number */
+    New(0, ctx.factors, square_free ? 15 : 63, UV);
+    ctx._nfactors = 0;
+    ctx._farray = ctx.factors;
+    ctx._noffset = 0;
+  }
+  return ctx;
+}
+
+int factor_range_next(factor_range_context_t *ctx) {
+  int j, nfactors;
+  UV n;
+  if (ctx->n >= ctx->hi)
+    return -1;
+  n = ++(ctx->n);
+  if (ctx->_nfactors) {
+    if (ctx->_coffset >= _fr_chunk) {
+      UV clo = n;
+      UV chi = n + _fr_chunk - 1;
+      if (chi > ctx->hi) chi = ctx->hi;
+      _vec_factor(clo, chi, ctx->_nfactors, ctx->_farray, ctx->_noffset, ctx->is_square_free);
+      ctx->_coffset = 0;
+    }
+    nfactors = ctx->_nfactors[ctx->_coffset];
+    ctx->factors = ctx->_farray + ctx->_coffset * ctx->_noffset;
+    ctx->_coffset++;
+  } else {
+    if (ctx->is_square_free && n >= 49 && (!(n% 4) || !(n% 9) || !(n%25) || !(n%49)))
+      return 0;
+    nfactors = factor(n, ctx->factors);
+    if (ctx->is_square_free) {
+      for (j = 1; j < nfactors; j++)
+        if (ctx->factors[j] == ctx->factors[j-1])
+          break;
+      if (j < nfactors) return 0;
+    }
+  }
+  return nfactors;
+}
+
+void factor_range_destroy(factor_range_context_t *ctx) {
+  if (ctx->_farray != 0) Safefree(ctx->_farray);
+  if (ctx->_nfactors != 0) Safefree(ctx->_nfactors);
+  ctx->_farray = ctx->_nfactors = ctx->factors = 0;
+}
+
+
+/******************************************************************************/
+/* DLP */
+/******************************************************************************/
+
 static UV dlp_trial(UV a, UV g, UV p, UV maxrounds) {
   UV k, t;
   if (maxrounds > p) maxrounds = p;
@@ -1456,7 +1816,7 @@ UV znlog(UV a, UV g, UV p) {
 
 
 /* Compile with:
- *  gcc -O3 -fomit-frame-pointer -march=native -Wall -DFACTOR_STANDALONE -DSTANDALONE factor.c util.c sieve.c cache.c primality.c lmo.c -lm
+ *  gcc -O3 -fomit-frame-pointer -march=native -Wall -DSTANDALONE -DFACTOR_STANDALONE factor.c util.c primality.c cache.c sieve.c chacha.c csprng.c prime_nth_count.c lmo.c -lm
  */
 #ifdef FACTOR_STANDALONE
 #include <errno.h>
@@ -1466,7 +1826,26 @@ int main(int argc, char *argv[])
   UV factors[MPU_MAX_FACTORS+1];
   int nfactors, i, a;
 
-  if (argc <= 1) { printf("usage: %s  <n>\n", argv[0]); return(1); }
+  if (argc <= 1) {
+    char line[1024];
+    while (1) {
+      if (!fgets(line,sizeof(line),stdin)) break;
+      n = strtoull(line, 0, 10);
+      nfactors = factor(n, factors);
+      if (nfactors == 1) {
+        printf("%"UVuf": %"UVuf"\n",n,n);
+      } else if (nfactors == 2) {
+        printf("%"UVuf": %"UVuf" %"UVuf"\n",n,factors[0],factors[1]);
+      } else if (nfactors == 3) {
+        printf("%"UVuf": %"UVuf" %"UVuf" %"UVuf"\n",n,factors[0],factors[1],factors[2]);
+      } else {
+        printf("%"UVuf": %"UVuf" %"UVuf" %"UVuf" %"UVuf"",n,factors[0],factors[1],factors[2],factors[3]);
+        for (i = 4; i < nfactors; i++) printf(" %"UVuf"", factors[i]);
+        printf("\n");
+      }
+    }
+    exit(0);
+  }
 
   for (a = 1; a < argc; a++) {
     n = strtoul(argv[a], 0, 10);
