@@ -102,8 +102,8 @@ typedef   signed long IV;
 
 static UV isqrt(UV n) {
   UV root;
-  if (sizeof(UV) == 8 && n >= 18446744065119617025UL)  return 4294967295UL;
-  if (sizeof(UV) == 4 && n >= 4294836225UL)            return 65535UL;
+  if (sizeof(UV)==8 && n >= UVCONST(18446744065119617025))  return 4294967295UL;
+  if (sizeof(UV)==4 && n >= 4294836225UL)            return 65535UL;
   root = (UV) sqrt((double)n);
   while (root*root > n)  root--;
   while ((root+1)*(root+1) <= n)  root++;
@@ -113,7 +113,7 @@ static UV icbrt(UV n) {
   UV b, root = 0;
   int s;
   if (sizeof(UV) == 8) {
-    s = 63;  if (n >= 18446724184312856125UL)  return 2642245UL;
+    s = 63;  if (n >= UVCONST(18446724184312856125))  return 2642245UL;
   } else {
     s = 30;  if (n >= 4291015625UL)            return 1625UL;
   }
@@ -137,7 +137,7 @@ static UV icbrt(UV n) {
   #include <omp.h>
 #endif
 
-#define _XS_prime_count(a, b)     primesieve::parallel_count_primes(a, b)
+#define segment_prime_count(a, b)     primesieve::parallel_count_primes(a, b)
 
 /* Generate an array of n small primes, where the kth prime is element p[k].
  * Remember to free when done. */
@@ -173,6 +173,7 @@ static uint32_t* generate_small_primes(UV n)
 #include "util.h"
 #include "cache.h"
 #include "sieve.h"
+#include "prime_nth_count.h"
 
 /* Generate an array of n small primes, where the kth prime is element p[k].
  * Remember to free when done. */
@@ -215,7 +216,7 @@ static UV bs_prime_count(uint32_t n, uint32_t const* const primes, uint32_t last
   UV i, j;
   if (n <= 2)  return (n == 2);
   /* If n is out of range, we could:
-   *  1. return _XS_prime_count(2, n);
+   *  1. return segment_prime_count(2, n);
    *  2. if (n == primes[lastidx]) return lastidx else croak("bspc range");
    *  3. if (n >= primes[lastidx]) return lastidx;
    */
@@ -236,7 +237,7 @@ static UV bs_prime_count(uint32_t n, uint32_t const* const primes, uint32_t last
     if (primes[mid] <= n)  i = mid+1;
     else                   j = mid;
   }
-  /* if (i-1 != _XS_prime_count(2, n)) croak("wrong count for %lu: %lu vs. %lu\n", n, i-1, _XS_prime_count(2, n)); */
+  /* if (i-1 != segment_prime_count(2, n)) croak("wrong count for %lu: %lu vs. %lu\n", n, i-1, segment_prime_count(2, n)); */
   return i-1;
 }
 
@@ -619,7 +620,7 @@ static UV phi(UV x, UV a)
 }
 
 
-extern UV _XS_meissel_pi(UV n);
+extern UV meissel_prime_count(UV n);
 /* b = prime_count(isqrt(n)) */
 static UV Pk_2_p(UV n, UV a, UV b, const uint32_t* primes, uint32_t lastidx)
 {
@@ -633,7 +634,7 @@ static UV Pk_2_p(UV n, UV a, UV b, const uint32_t* primes, uint32_t lastidx)
   for (i = b; i > a; i--) {
     UV w = n / primes[i];
     lastwpc = (w <= lastpc) ? bs_prime_count(w, primes, lastidx)
-                            : lastwpc + _XS_prime_count(lastw+1, w);
+                            : lastwpc + segment_prime_count(lastw+1, w);
     lastw = w;
     P2 += lastwpc;
   }
@@ -652,13 +653,13 @@ static UV Pk_2(UV n, UV a, UV b)
 
 /* Legendre's method.  Interesting and a good test for phi(x,a), but Lehmer's
  * method is much faster (Legendre: a = pi(n^.5), Lehmer: a = pi(n^.25)) */
-UV _XS_legendre_pi(UV n)
+UV legendre_prime_count(UV n)
 {
   UV a, phina;
   if (n < SIEVE_LIMIT)
-    return _XS_prime_count(2, n);
+    return segment_prime_count(2, n);
 
-  a = _XS_legendre_pi(isqrt(n));
+  a = legendre_prime_count(isqrt(n));
   /* phina = phi(n, a); */
   { /* The small phi routine is faster for large a */
     cache_t pcache;
@@ -674,14 +675,14 @@ UV _XS_legendre_pi(UV n)
 
 
 /* Meissel's method. */
-UV _XS_meissel_pi(UV n)
+UV meissel_prime_count(UV n)
 {
   UV a, b, sum;
   if (n < SIEVE_LIMIT)
-    return _XS_prime_count(2, n);
+    return segment_prime_count(2, n);
 
-  a = _XS_meissel_pi(icbrt(n));       /* a = Pi(floor(n^1/3)) [max    192725] */
-  b = _XS_meissel_pi(isqrt(n));       /* b = Pi(floor(n^1/2)) [max 203280221] */
+  a = meissel_prime_count(icbrt(n));       /* a = Pi(floor(n^1/3)) [max    192725] */
+  b = meissel_prime_count(isqrt(n));       /* b = Pi(floor(n^1/2)) [max 203280221] */
 
   sum = phi(n, a) + a - 1 - Pk_2(n, a, b);
   return sum;
@@ -689,29 +690,29 @@ UV _XS_meissel_pi(UV n)
 
 /* Lehmer's method.  This is basically Riesel's Lehmer function (page 22),
  * with some additional code to help optimize it.  */
-UV _XS_lehmer_pi(UV n)
+UV lehmer_prime_count(UV n)
 {
   UV z, a, b, c, sum, i, j, lastprime, lastpc, lastw, lastwpc;
   const uint32_t* primes = 0; /* small prime cache, first b=pi(z)=pi(sqrt(n)) */
   DECLARE_TIMING_VARIABLES;
 
   if (n < SIEVE_LIMIT)
-    return _XS_prime_count(2, n);
+    return segment_prime_count(2, n);
 
   /* Protect against overflow.  2^32-1 and 2^64-1 are both divisible by 3. */
   if (n == UV_MAX) {
     if ( (n%3) == 0 || (n%5) == 0 || (n%7) == 0 || (n%31) == 0 )
       n--;
     else
-      return _XS_prime_count(2,n);
+      return segment_prime_count(2,n);
   }
 
   if (verbose > 0) printf("lehmer %lu stage 1: calculate a,b,c \n", n);
   TIMING_START;
   z = isqrt(n);
-  a = _XS_lehmer_pi(isqrt(z));        /* a = Pi(floor(n^1/4)) [max      6542] */
-  b = _XS_lehmer_pi(z);               /* b = Pi(floor(n^1/2)) [max 203280221] */
-  c = _XS_lehmer_pi(icbrt(n));        /* c = Pi(floor(n^1/3)) [max    192725] */
+  a = lehmer_prime_count(isqrt(z));        /* a = Pi(floor(n^1/4)) [max      6542] */
+  b = lehmer_prime_count(z);               /* b = Pi(floor(n^1/2)) [max 203280221] */
+  c = lehmer_prime_count(icbrt(n));        /* c = Pi(floor(n^1/3)) [max    192725] */
   TIMING_END_PRINT("stage 1")
 
   if (verbose > 0) printf("lehmer %lu stage 2: phi(x,a) (z=%lu a=%lu b=%lu c=%lu)\n", n, z, a, b, c);
@@ -746,7 +747,7 @@ UV _XS_lehmer_pi(UV n)
   for (i = b; i >= a+1; i--) {
     UV w = n / primes[i];
     lastwpc = (w <= lastpc) ? bs_prime_count(w, primes, lastprime)
-                            : lastwpc + _XS_prime_count(lastw+1, w);
+                            : lastwpc + segment_prime_count(lastw+1, w);
     lastw = w;
     sum = sum - lastwpc;
     if (i <= c) {
@@ -768,7 +769,7 @@ UV _XS_lehmer_pi(UV n)
  * About the same speed as Lehmer, a bit less memory.
  * A better implementation can be 10-50x faster and much less memory.
  */
-UV _XS_LMOS_pi(UV n)
+UV LMOS_prime_count(UV n)
 {
   UV n13, a, b, sum, i, j, k, lastprime, P2, S1, S2;
   const uint32_t* primes = 0;  /* small prime cache */
@@ -778,11 +779,11 @@ UV _XS_LMOS_pi(UV n)
   DECLARE_TIMING_VARIABLES;
 
   if (n < SIEVE_LIMIT)
-    return _XS_prime_count(2, n);
+    return segment_prime_count(2, n);
 
   n13 = icbrt(n);                    /* n13 =  floor(n^1/3)  [max    2642245] */
-  a = _XS_lehmer_pi(n13);            /* a = Pi(floor(n^1/3)) [max     192725] */
-  b = _XS_lehmer_pi(isqrt(n));       /* b = Pi(floor(n^1/2)) [max  203280221] */
+  a = lehmer_prime_count(n13);       /* a = Pi(floor(n^1/3)) [max     192725] */
+  b = lehmer_prime_count(isqrt(n));  /* b = Pi(floor(n^1/2)) [max  203280221] */
 
   lastprime = b*SIEVE_MULT+1;
   if (lastprime > 203280221) lastprime = 203280221;
@@ -865,11 +866,11 @@ int main(int argc, char *argv[])
 
   gettimeofday(&t0, 0);
 
-  if      (!strcasecmp(method, "lehmer"))   { pi = _XS_lehmer_pi(n);      }
-  else if (!strcasecmp(method, "meissel"))  { pi = _XS_meissel_pi(n);     }
-  else if (!strcasecmp(method, "legendre")) { pi = _XS_legendre_pi(n);    }
-  else if (!strcasecmp(method, "lmo"))      { pi = _XS_LMOS_pi(n);  }
-  else if (!strcasecmp(method, "sieve"))    { pi = _XS_prime_count(2, n); }
+  if      (!strcasecmp(method, "lehmer"))   { pi = lehmer_prime_count(n);      }
+  else if (!strcasecmp(method, "meissel"))  { pi = meissel_prime_count(n);     }
+  else if (!strcasecmp(method, "legendre")) { pi = legendre_prime_count(n);    }
+  else if (!strcasecmp(method, "lmo"))      { pi = LMOS_prime_count(n);  }
+  else if (!strcasecmp(method, "sieve"))    { pi = segment_prime_count(2, n); }
   else {
     printf("method must be one of: lehmer, meissel, legendre, lmo, or sieve\n");
     return(2);
@@ -884,9 +885,9 @@ int main(int argc, char *argv[])
 #else
 
 #include "lehmer.h"
-UV _XS_LMOS_pi(UV n)      { if (n==0) return 0; croak("Not compiled with Lehmer support"); }
-UV _XS_lehmer_pi(UV n)    { if (n==0) return 0; croak("Not compiled with Lehmer support"); }
-UV _XS_meissel_pi(UV n)   { if (n==0) return 0; croak("Not compiled with Lehmer support"); }
-UV _XS_legendre_pi(UV n)  { if (n==0) return 0; croak("Not compiled with Lehmer support"); }
+UV LMOS_prime_count(UV n)      { if (n!=0) croak("Not compiled with Lehmer support"); return 0;}
+UV lehmer_prime_count(UV n)    { if (n!=0) croak("Not compiled with Lehmer support"); return 0;}
+UV meissel_prime_count(UV n)   { if (n!=0) croak("Not compiled with Lehmer support"); return 0;}
+UV legendre_prime_count(UV n)  { if (n!=0) croak("Not compiled with Lehmer support"); return 0;}
 
 #endif
