@@ -53,13 +53,18 @@ typedef __int8 int8_t;
   #include <ctype.h>
   typedef unsigned long UV;
   typedef   signed long IV;
-  typedef size_t        STRLEN;
+  typedef        double NV;
+  typedef        size_t STRLEN;
   #define UV_MAX ULONG_MAX
   #define IV_MAX LONG_MAX
+  #define NV_MAX DBL_MAX
   #define UVCONST(x) ((unsigned long)x##UL)
   #define U32_CONST(x) ((unsigned int)x##U)
   #define UVuf "lu"
   #define IVdf "ld"
+  #define NVff "f"
+  /* Technically this is sizeof(NV) but that's not valid for macros */
+  #define NVSIZE 8
   #define croak(fmt,...)            { printf(fmt,##__VA_ARGS__); exit(3); }
   #define New(id, mem, size, type)  mem = (type*) malloc((size)*sizeof(type))
   #define Newz(id, mem, size, type) mem = (type*) calloc(size, sizeof(type))
@@ -116,14 +121,10 @@ typedef __int8 int8_t;
   #define UVCONST(x)     U32_CONST(x)
 #endif
 
+#endif   /* End of Perl specific section */
+
 /* Try to determine if we have 64-bit available via uint64_t */
-#if defined(UINT64_MAX) && defined(__UINT64_C)
-  #if (UINT64_MAX >= __UINT64_C(18446744073709551615))
-    #define HAVE_STD_U64 1
-  #else
-    #define HAVE_STD_U64 0
-  #endif
-#elif defined(_UINT64_T) /* Darwin Clang */
+#if defined(UINT64_MAX) || defined(_UINT64_T) || defined(__UINT64_TYPE__)
   #define HAVE_STD_U64 1
 #elif defined(_MSC_VER)   /* We set up the types earlier */
   #define HAVE_STD_U64 1
@@ -131,13 +132,13 @@ typedef __int8 int8_t;
   #define HAVE_STD_U64 0
 #endif
 
-#endif
-
 #define MAXBIT        (BITS_PER_WORD-1)
 #define NWORDS(bits)  ( ((bits)+BITS_PER_WORD-1) / BITS_PER_WORD )
 #define NBYTES(bits)  ( ((bits)+8-1) / 8 )
 
 #define MPUassert(c,text) if (!(c)) { croak("Math::Prime::Util internal error: " text); }
+#define MPUverbose(level,fmt,...) \
+  if (_XS_get_verbose() >= level) { printf(fmt,##__VA_ARGS__); fflush(stdout); }
 
 /* The ASSUME bits are from perl 5.19.6 perl.h */
 
@@ -163,8 +164,14 @@ typedef __int8 int8_t;
 
 #define MPUNOT_REACHED MPUASSUME(0)
 
-#if (defined(__SIZEOF_INT128__) || __GNUC__ >= 4) && (defined(__x86_64__) || defined(__powerpc64__))
-  #if __GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ < 4)
+#if defined(__SIZEOF_INT128__) && !defined(__CUDACC__)
+  #define HAVE_UINT128 1
+  typedef unsigned __int128 uint128_t;
+#elif (__GNUC__ >= 4) && (defined(__x86_64__) || defined(__powerpc64__))
+  #if __clang__ && (__clang_major__ > 4 || (__clang_major__ == 4 && __clang_minor__ >= 2))
+    #define HAVE_UINT128 1
+    typedef unsigned __int128 uint128_t;
+  #elif __GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ < 4)
     #define HAVE_UINT128 0
   #elif __GNUC__ == 4 && __GNUC_MINOR__ >= 4 && __GNUC_MINOR__ < 6
     #define HAVE_UINT128 1
@@ -177,7 +184,44 @@ typedef __int8 int8_t;
   #define HAVE_UINT128 0
 #endif
 
-#if defined(__GNUC__)
+/* Perl 5.23.0 added the very helpful definition.  Without it, guess. */
+#ifndef NVMANTBITS
+  #if NVSIZE <= 8
+    #define NVMANTBITS ((NVSIZE <= 4) ? 24 : 53)
+  #elif defined(USE_QUADMATH)
+    #define NVMANTBITS 112
+  #elif defined(__LDBL_MANT_DIG__)
+    #define NVMANTBITS __LDBL_MANT_DIG__
+  #else
+    #define NVMANTBITS 64
+  #endif
+#endif
+
+#if defined(USE_QUADMATH)
+  typedef __float128   LNV;
+  #define LNV_ZERO     0.0Q
+  #define LNV_ONE      1.0Q
+  #define LNVCONST(x) ((__float128)x##Q)
+  #define loglnv(x)    logq(x)
+  #define explnv(x)    expq(x)
+  #define sqrtlnv(x)   sqrtq(x)
+  #define fabslnv(x)   fabsq(x)
+  #define LNV_EPSILON  FLT128_EPSILON
+  #define LNV_IS_QUAD  1
+#else
+  typedef long double  LNV;
+  #define LNV_ZERO     0.0L
+  #define LNV_ONE      1.0L
+  #define LNVCONST(x) ((long double)x##L)
+  #define loglnv(x)    logl(x)
+  #define explnv(x)    expl(x)
+  #define sqrtlnv(x)   sqrtl(x)
+  #define fabslnv(x)   fabsl(x)
+  #define LNV_EPSILON  LDBL_EPSILON
+  #define LNV_IS_QUAD  0
+#endif
+
+#if defined(__GNUC__) || defined(__clang__)
   #define INLINE inline
 #elif defined(_MSC_VER)
   #define INLINE __inline
@@ -185,4 +229,39 @@ typedef __int8 int8_t;
   #define INLINE
 #endif
 
+#if __BIG_ENDIAN__ || (defined(BYTEORDER) && (BYTEORDER == 0x4321 || BYTEORDER == 0x87654321))
+#  if (__GNUC__ == 4 && __GNUC_MINOR__ >= 4) || __GNUC__ >= 5 || (__clang__ &&  __clang_major__ >= 4)
+#    if BITS_PER_WORD == 64
+#      define LEUV(x) __builtin_bswap64(x)
+#    else
+#      define LEUV(x) __builtin_bswap32(x)
+#    endif
+#  else
+#    if BITS_PER_WORD == 64
+       /* compare to 5 step interleave */
+       static UV LEUV(UV x) {
+         UV v = ((x & UVCONST(0xFF00000000000000)) >> 56) |
+                ((x & UVCONST(0x00FF000000000000)) >> 40) |
+                ((x & UVCONST(0x0000FF0000000000)) >> 24) |
+                ((x & UVCONST(0x000000FF00000000)) >>  8) |
+                ((x & UVCONST(0x00000000FF000000)) <<  8) |
+                ((x & UVCONST(0x0000000000FF0000)) << 24) |
+                ((x & UVCONST(0x000000000000FF00)) << 40) |
+                ((x & UVCONST(0x00000000000000FF)) << 56);
+         return v;
+       }
+#    else
+       static UV LEUV(UV x) {
+         UV v = ((x & 0xFF000000) >> 24) |
+                ((x & 0x00FF0000) >>  8) |
+                ((x & 0x0000FF00) <<  8) |
+                ((x & 0x000000FF) << 24);
+         return v;
+       }
+#    endif
+#  endif
+#else /* LE */
+#  define LEUV(x) (x)
+#endif
+
 #endif
